#pragma once

#include <Windows.h>
#include "UsbKeys.h"

#include <pshpack1.h>
struct HIDINJECTOR_INPUT_REPORT
{
	unsigned char ReportId;
	union {
		struct {
			UCHAR Modifiers;
			UCHAR Key1;
			UCHAR Key2;
			UCHAR Key3;
			UCHAR Key4;
			UCHAR Padding[2]; // Padding to keep the reports the same size
		} KeyReport;
		struct {
			UCHAR Buttons;
			SHORT AbsoluteX;
			SHORT AbsoluteY;
			UCHAR Padding[2]; // Padding to keep the reports the same size
		} MouseReport;
		struct {
			UCHAR Flags;  // Bit 0 = Tip Switch, Bit 1 = In Range, Bit 2-7 = unused
			UCHAR ContactIndentifier;
			USHORT AbsoluteX;
			USHORT AbsoluteY;
			UCHAR ContactCount;
		} TouchReport;
	} Report;
};

// Values for ReportId
#define KEYBOARD_REPORT_ID		1
#define MOUSE_REPORT_ID			2
#define TOUCH_REPORT_ID			3
#define MAX_COUNT_REPORT_ID		4

// Values for MouseReport.Buttons from the HID Spec
#define MOUSE_BUTTON_1			0x01
#define MOUSE_BUTTON_2			0x02
#define MOUSE_BUTTON_3			0x04

// Values for TouchReport.Flags from the HID Spec
#define TOUCH_TIP_SWITCH		0x01
#define TOUCH_IN_RANGE			0x02
#define TOUCH_MAX_FINGER		0x0a // 10
#define TOUCH_PHYSICAL_MAX		0x7FFF	//Physical maximum given for the X and Y usage at the touch part of the HID_REPORT_DESCRIPTOR at the driver.

#include <poppack.h>

using namespace HidDevice;

struct TouchData
{
	BOOL down;
	int x;
	int y;
};

class HidInjector
{
private:
	static int hcount;
	static HANDLE handle;

	static HIDINJECTOR_INPUT_REPORT mouse_state;
	static HIDINJECTOR_INPUT_REPORT key_state;
	static HIDINJECTOR_INPUT_REPORT touch_state;

	static BOOL mouseUsed;
	static TouchData touches[TOUCH_MAX_FINGER];

	static int sc_width, sc_height;

	void OutputMessage(LPCWSTR msg, ...);

	BOOL FindMatchingDevice();
	BOOL SendReport(HIDINJECTOR_INPUT_REPORT *report);

	BOOL SendKey(int pos, UsbKeys uk);
	BOOL SendMouse(int x, int y, UCHAR buttons);
	BOOL SendTouch(int pos);

public:
	HidInjector();
	~HidInjector();

	BOOL Open();
	void Close();

	BOOL TouchDown(UCHAR pos);
	BOOL TouchMove(UCHAR pos, int x, int y);
	BOOL TouchUp(UCHAR pos);

	BOOL KeyDown(int pos, UsbKeys uk);
	BOOL KeyUp(int pos);

	BOOL MouseMove(int x, int y);
	BOOL MouseDown(UCHAR pos);
	BOOL MouseUp(UCHAR pos);

	static int GetScreenWidth() { return sc_width; }
	static int GetScreenHeight() { return sc_height; }
};