#include "HidInjector.h"
#include <collection.h>
#include <ppltasks.h>

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Devices;
using namespace Windows::Devices::Gpio;
using namespace Windows::Devices::Spi;
using namespace Windows::Devices::Enumeration;

#define SPI_DEVICE_ID "\\\\?\\ACPI#MSFT8000#1#{dcde6af9-6610-4285-828f-caaf78c424cc}\\SPI0"
const wchar_t waitName[] = L"thit-wait";
const wchar_t mutexName[] = L"thit-mutex";

HidInjector hi;
HANDLE waitHandle = INVALID_HANDLE_VALUE;

void OnValueChanged(GpioPin ^pin, GpioPinValueChangedEventArgs ^args)
{
	if (pin->Read() == GpioPinValue::High) {
#ifdef MOUSE_TEST
		hi.MouseDown(MOUSE_BUTTON_2);
#else
		hi.TouchDown(0);
#endif
	} else {
#ifdef MOUSE_TEST
		hi.MouseUp(MOUSE_BUTTON_2);
#else
		hi.TouchUp(0);
#endif
	}
}

void InputAsync()
{
	// Send "tnksoft" keystroke
	UsbKeys uks[] = { UsbKeys::T, UsbKeys::N, UsbKeys::K, UsbKeys::S, UsbKeys::O, UsbKeys::F, UsbKeys::T };
	for (int i = 0; i < 7; i++) {
		hi.KeyDown(0, uks[i]);
		hi.KeyUp(0);
	}

	auto settings = ref new SpiConnectionSettings(0);
	settings->ClockFrequency = 5000000;
	settings->Mode = SpiMode::Mode3;
	concurrency::create_task(SpiDevice::FromIdAsync(SPI_DEVICE_ID, settings)).then([](SpiDevice ^spi)->BOOL{
		auto gpio = GpioController::GetDefault();
		auto pin = gpio->OpenPin(6);
		pin->SetDriveMode(GpioPinDriveMode::InputPullDown);
		pin->ValueChanged += ref new TypedEventHandler<GpioPin ^, GpioPinValueChangedEventArgs ^>(&OnValueChanged);

		auto spc = gpio->OpenPin(5);
		spc->SetDriveMode(GpioPinDriveMode::Output);

		Array<byte> ^sendBuf = ref new Array<byte>(2) { 0b01'10'1'000, 0x00 };
		Array<byte> ^recvBuf = ref new Array<byte>(2);

		int dx = 0, dy = HidInjector::GetScreenHeight() / 2, v;

		while (waitHandle != INVALID_HANDLE_VALUE) {
			Sleep(10);

			spc->Write(GpioPinValue::Low);
			spi->TransferFullDuplex(sendBuf, recvBuf);
			spc->Write(GpioPinValue::High);

			v = (recvBuf[0] << 8) | recvBuf[1];
			dx = 100 + (HidInjector::GetScreenWidth() - 200) * v / 1023;

			// If you want to get Y position.
			/*
			Array<byte> ^sendBufy = ref new Array<byte>(2) { 0b01'11'1'000, 0x00 };
			spc->Write(GpioPinValue::Low);
			spi->TransferFullDuplex(sendBufy, recvBuf);
			spc->Write(GpioPinValue::High);
			dy = 100 + (HidInjector::GetScreenHeight() - 200) * v / 1023;
			*/

#ifdef MOUSE_TEST
			hi.MouseMove(dx, dy);
#else
			hi.TouchMove(0, dx, dy);
#endif
		}

		return TRUE;
	});
}

int main(Platform::Array<Platform::String^>^ args)
{
	HANDLE mutex = CreateMutex(NULL, FALSE, mutexName);
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		if (args[1] == "e") {
			waitHandle = OpenEvent(EVENT_MODIFY_STATE, FALSE, waitName);
			if(waitHandle != NULL) SetEvent(waitHandle);
		}
		return 1;
	}

	if (hi.Open() == FALSE) return 1;

	waitHandle = CreateEvent(NULL, TRUE, FALSE, waitName);
	concurrency::create_task(InputAsync);

	WaitForSingleObject(waitHandle, INFINITE);
	CloseHandle(waitHandle);
	waitHandle = INVALID_HANDLE_VALUE;

	hi.Close();

	return 0;
}
